{% set not_defined_icon %}
<span style="color:#9178C600; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Input"><span class="twemoji"> <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M4 15V9h8V4.16L19.84 12 12 19.84V15z"></path></svg></span></span>
{% endset %}

{% set defined_icon %}
<span style="color:#9178C6; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Input" class="twemoji">{{ ":material-arrow-right-bold:"|convert_markdown_simple(0) }}</span>
{% endset %}

{% set show_icon %}
<span style="color:#52BF54; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Shown" class="twemoji">{{ ":material-eye:"|convert_markdown_simple(0) }}</span>
{% endset %}

{% set show_explicit_icon %}
<span style="color:#52BF54; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Shown Explicitly" class="twemoji">{{ ":material-eye:"|convert_markdown_simple(0) }}</span>
{% endset %}

{% set show_partial_icon %}
<span style="color:#52BF54; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Shown Partially" class="twemoji">{{ ":material-eye-plus:"|convert_markdown_simple(0) }}</span>
{% endset %}

{% set hidden_icon %}
<span style="color:#88888850; padding-top: 0; padding-bottom: 0; font-size:0.45rem" title="Hidden" class="twemoji">{{ ":material-eye-off:"|convert_markdown_simple(0) }}</span>
{% endset %}


{% set show_icon_map = {
    0: show_icon,
    1: show_explicit_icon,
    2: show_partial_icon,
    3: show_explicit_icon + show_partial_icon,
    4: hidden_icon
} %}

{% set input_icon_map = {
    0: not_defined_icon,
    1: defined_icon,
} %}
